#redefine vals to fit in specific situation
#define NN shape-----------------------------------
NN_INPUT_SIZE = 32
NN_OUTPUT_SIZE = 4
NN_HIDDENLAYER_NUM = 2
NN_HIDDEN_SIZE = 18
#define AG coefs-----------------------------------
AG_POP_SIZE = 100#种群数
AG_POP_PM = 0.1#变异率
AG_POP_PC = 0.7#交叉率
