from setuptools import setup, find_packages
setup(
    name = 'gameai_frame',\
    version = '0.1.3',\
    description = ('maybe a generitic game ai frame :)'),\
    long_description = open('E:\Python\PY_APP6\gameai_frame\README.md', 'r', encoding='UTF-8').read(),\
    author = 'Ehekatl',\
    author_email = '2072498313@qq.com',\
    maintainer = 'Ehekatl',\
    maintainer_email = '2072498313@qq.com',\
    license = 'MIT License',\
    packages = find_packages(),\
    platforms = ['all'],\
    url = 'https://github.com/Iyrian/GameAI-Frame-',\
    install_requires = ['numpy', 'tkinter'],\
    classifiers = ['Programming Language :: Python :: 3.6', 'Topic :: Software Development :: Libraries'])